package controllers

import (
	"encoding/json"
	"eta/eta_mini_bridge/models"
	"eta/eta_mini_bridge/utils"
	"fmt"
	"net/http"
	"net/url"

	"github.com/beego/beego/v2/server/web"
)

type BaseAuthController struct {
	web.Controller
	Token string
}

func (this *BaseAuthController) Prepare() {
	method := this.Ctx.Input.Method()
	uri := this.Ctx.Input.URI()
	fmt.Println("Url:", uri)

	header := this.Ctx.Request.Header
	headerBody, err := json.Marshal(header)
	fmt.Println(err)
	fmt.Println("header:" + string(headerBody))

	if method != "HEAD" {
		//校验签名
		nonce := this.Ctx.Input.Header("Nonce")
		timestamp := this.Ctx.Input.Header("Timestamp")
		appid := this.Ctx.Input.Header("Appid")
		signature := this.Ctx.Input.Header("Signature")

		if nonce == "" {
			errMsg := "随机字符串不能为空"
			this.JSON(models.BaseResponse{Ret: 400, Msg: "", ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}

		if timestamp == "" {
			errMsg := "时间戳不能为空"
			this.JSON(models.BaseResponse{Ret: 400, Msg: "", ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}

		checkSign := utils.GetSign(nonce, timestamp, appid, utils.ETA_MINI_API_SECRET)
		if signature != checkSign {
			errMsg := "签名错误"
			this.JSON(models.BaseResponse{Ret: 401, Msg: "", ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}
		if method != "GET" && method != "POST" {
			errMsg := "无效的请求方式"
			this.JSON(models.BaseResponse{Ret: 501, Msg: "", ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}
	} else {
		this.JSON(models.BaseResponse{Ret: 500, Msg: "系统异常，请联系客服!", ErrMsg: "method:" + method}, false, false)
		this.StopRun()
		return
	}
}

func (c *BaseAuthController) ServeJSON(encoding ...bool) {
	//所有请求都做这么个处理吧，目前这边都是做编辑、刷新逻辑处理（新增的话，并没有指标id，不会有影响）
	var (
		hasIndent   = false
		hasEncoding = false
	)
	if web.BConfig.RunMode == web.PROD {
		hasIndent = false
	}
	if len(encoding) > 0 && encoding[0] == true {
		hasEncoding = true
	}
	if c.Data["json"] == nil {
		utils.ApiLog.Info("异常提醒:", "接口:"+"URI:"+c.Ctx.Input.URI()+";无返回值")
		return
	}

	baseRes := c.Data["json"].(*models.BaseResponse)
	if baseRes != nil && baseRes.Ret != 408 {
		//body, _ := json.Marshal(baseRes)
		//var requestBody string
		//method := c.Ctx.Input.Method()
		//if method == "GET" {
		//	requestBody = c.Ctx.Request.RequestURI
		//} else {
		//	requestBody, _ = url.QueryUnescape(string(c.Ctx.Input.RequestBody))
		//}
		//if baseRes.Ret != 200 && baseRes.IsSendEmail {
		//	go utils.SendEmail(utils.APP_NAME_CN+"【"+utils.RunMode+"】"+"失败提醒", "URI:"+c.Ctx.Input.URI()+"<br/> "+"Params"+requestBody+" <br/>"+"ErrMsg:"+baseRes.ErrMsg+";<br/>Msg:"+baseRes.Msg+";<br/> Body:"+string(body)+"<br/>", utils.EmailSendToUsers)
		//}

		// 记录错误日志, 并清掉错误信息避免暴露给外部
		if baseRes.ErrMsg != "" {

			utils.ApiLog.Info(baseRes.ErrMsg)
			baseRes.ErrMsg = ""
		}
	}
	c.JSON(c.Data["json"], hasIndent, hasEncoding)
}

func (c *BaseAuthController) JSON(data interface{}, hasIndent bool, coding bool) error {
	c.Ctx.Output.Header("Content-Type", "application/json; charset=utf-8")
	var content []byte
	var err error
	if hasIndent {
		content, err = json.MarshalIndent(data, "", "  ")
	} else {
		content, err = json.Marshal(data)
	}
	if err != nil {
		http.Error(c.Ctx.Output.Context.ResponseWriter, err.Error(), http.StatusInternalServerError)
		return err
	}
	ip := c.Ctx.Input.IP()
	requestBody, err := url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	if err != nil {
		requestBody = string(c.Ctx.Input.RequestBody)
	}
	if requestBody == "" {
		requestBody = c.Ctx.Input.URI()
	}
	c.logUri(data, requestBody, ip)
	if coding {
		content = []byte(utils.StringsToJSON(string(content)))
	}
	return c.Ctx.Output.Body(content)
}

func (c *BaseAuthController) logUri(respContent interface{}, requestBody, ip string) {
	utils.ApiLog.Info("uri:%s,  requestBody:%s, responseBody:%s, ip:%s", c.Ctx.Input.URI(), requestBody, respContent, ip)
	return
}
