package controllers

import (
	"eta/eta_mini_bridge/models"
	"eta/eta_mini_bridge/models/response"
	"eta/eta_mini_bridge/utils"

	"github.com/rdlucklib/rdluck_tools/paging"
)

type ChartController struct {
	BaseAuthController
}

// @Title List
// @Description create users
// @Param   PageSize   query   int  true       "每页数据条数"
// @Param   CurrentIndex   query   int  true       "当前页页码，从1开始"
// @Success 200 {object} models.BaseResponse
// @Failure 403 {object} models.BaseResponse
// @router /list [get]
func (this *ChartController) List() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	pageSize, _ := this.GetInt("PageSize")
	currentIndex, _ := this.GetInt("CurrentIndex")

	var startSize int
	if pageSize <= 0 {
		pageSize = utils.PageSize2
	}
	if currentIndex <= 0 {
		currentIndex = 1
	}
	startSize = utils.StartIndex(currentIndex, pageSize)
	total, err := models.GetChartCount()
	if err != nil {
		br.Msg = "获取图表列表失败"
		br.ErrMsg = "获取图表列表失败，系统错误，Err:" + err.Error()
		return
	}
	chartList, err := models.GetChartList(startSize, pageSize)
	if err != nil {
		br.Msg = "获取图表列表失败"
		br.ErrMsg = "获取图表列表失败，系统错误，Err:" + err.Error()
		return
	}
	page := paging.GetPaging(currentIndex, pageSize, total)
	resp := new(response.ChartListResp)
	resp.List = chartList
	resp.Paging = page

	br.Ret = 200
	br.Msg = "获取图表列表成功"
	br.Success = true
	br.Data = resp

}

// @Title Detail
// @Description 获得图表详情
// @Param   ChartInfoId   query   int  true       "图表详情id"
// @Param    UniqueCode  query   string  true       "图表唯一id"
// @Success 200 {object} models.BaseResponse
// @Failure 403 {object} models.BaseResponse
// @router /detail [get]
func (this *ChartController) Detail() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	chartInfoId, _ := this.GetInt("ChartInfoId")
	uniqueCode := this.GetString("UniqueCode")
	if chartInfoId <= 0 && uniqueCode == "" {
		br.Msg = "图表id错误"
		return
	}

	var chart *models.ChartInfoView
	var err error
	if chartInfoId > 0 {
		chart, err = models.GetChartById(chartInfoId)
	} else {
		chart, err = models.GetChartByUniqueCode(uniqueCode)
	}
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			item := new(models.ChartInfoView)
			br.Ret = 200
			br.Data = item
			br.Msg = "图表不存在"
			br.Success = true
			return
		}
		br.Msg = "获取图表详情失败"
		br.ErrMsg = "获取图表详情失败，系统错误，Err:" + err.Error()
		return
	}

	br.Ret = 200
	br.Msg = "获取图表成功"
	br.Success = true
	br.Data = chart
}
