package utils

import (
	"fmt"

	beeLogger "github.com/beego/bee/v2/logger"
	"github.com/beego/beego/v2/server/web"
)

// 数据库配置
var (
	RunMode          string // 运行模式
	MYSQL_URL_MASTER string // 数据库地址
	MYSQL_URL_RDDP   string // 数据库地址
	MYSQL_URL_DATA   string

	REDIS_CACHE string      //缓存地址
	Rc          RedisClient //redis缓存

)

var (
	LogPath    string //调用过程中的日志存放地址
	LogFile    string
	ApiLogPath string // 接口请求地址和接口返回值日志存放地址
	ApiLogFile string
	BinLogPath string // 数据库相关的日志存放地址
	BinLogFile string
	LogMaxDays int // 日志最大保留天数
)

// 小程序服务通信密钥
var (
	ETA_MINI_API_APPID  string
	ETA_MINI_API_SECRET string
)

// 报警地址
var AlarmMsgUrl string

// 微信相关
var (
	WX_MINI_APPID          string // 小程序appid
	WX_MINI_APP_SECRET     string
	DW_WX_Id               string //微信原始ID
	DW_WX_APPID            string // 东吴公众号的appid
	DW_WX_APP_SECRET       string
	TEMPLATE_ID_BY_PRODUCT string // 模板id
)

// ES配置
var (
	ES_URL      string // ES服务器地址
	ES_USERNAME string // ES账号
	ES_PASSWORD string // ES密码
)

// ES索引配置
var (
	EsReportIndexName              string //研报ES索引
	EsSemanticAnalysisDocIndexName string //ES语义分析文档索引名
	SmartReportIndexName           string //智能研报ES索引
)

func init() {
	tmpRunMode, err := web.AppConfig.String("run_mode")
	if err != nil {
		panic(any("配置文件读取run_mode错误 " + err.Error()))
	}
	RunMode = tmpRunMode
	fmt.Println("RunMode:", RunMode)
	if RunMode == "" {
		configPath := `/home/code/config/eta_mini_bridge/conf/app.conf`
		fmt.Println("configPath:", configPath)
		err = web.LoadAppConfig("ini", configPath)
		if err != nil {
			fmt.Println("web.LoadAppConfig Err:" + err.Error())
		}
		tmpRunMode, _ := web.AppConfig.String("run_mode")
		RunMode = tmpRunMode
	}

	config, err := web.AppConfig.GetSection(RunMode)
	if err != nil {
		panic(any("配置文件读取错误 " + err.Error()))
	}
	beeLogger.Log.Info(RunMode + " 模式")

	ETA_MINI_API_APPID = config["eta_mini_api_appid"]
	ETA_MINI_API_SECRET = config["eta_mini_api_secret"]
	// 数据库配置
	MYSQL_URL_RDDP = config["mysql_url_rddp"]
	MYSQL_URL_MASTER = config["mysql_url_master"]
	MYSQL_URL_DATA = config["mysql_url_data"]

	// 微信配置
	WX_MINI_APPID = config["wx_mini_appid"]
	WX_MINI_APP_SECRET = config["wx_mini_app_secret"]
	DW_WX_Id = config["dw_wx_id"]
	DW_WX_APPID = config["dw_wx_appid"]
	DW_WX_APP_SECRET = config["dw_wx_app_secret"]
	TEMPLATE_ID_BY_PRODUCT = config["template_id_by_product"]

	// redis缓存配置
	REDIS_CACHE = config["beego_cache"]
	if len(REDIS_CACHE) <= 0 {
		panic(any("redis链接参数没有配置"))
	}
	// 初始化缓存
	redisClient, err := initRedis(config["redis_type"], config["beego_cache"])
	if err != nil {
		fmt.Println("redis链接异常：", err)
		panic(any("redis链接参数没有配置"))
	}

	Rc = redisClient

	// ES配置
	{
		ES_URL = config["es_url"]
		ES_USERNAME = config["es_username"]
		ES_PASSWORD = config["es_password"]
	}
	// ES 索引
	{
		EsReportIndexName = config["es_report_index_name"]
		EsSemanticAnalysisDocIndexName = config["es_semantic_analysis_doc_index_name"]
		SmartReportIndexName = config["es_smart_report_index_name"]
	}
	initEs()

}
